import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import javax.swing.JOptionPane;
import java.net.*;

public class SudokuGame1
{	
	public Number[][] numbers = new Number[9][9];
	public KarrayList[][] quadrants = new KarrayList[3][3];
	public KarrayList[] rows = new KarrayList[9];
	public KarrayList[] columns = new KarrayList[9];
	public int overflow = 0;
	public Random rand = new Random();
	
	public SudokuGame1()
	{
		makeGame();
	}
	
	
	public void makeGame()
	{
		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 3; j++)
			{
				quadrants[j][i] = new KarrayList();
				for (int q = 1; q < 10; q++)
					quadrants[j][i].add(q); 
			}
		}
		
		for (int i = 0; i < 9; i++)
		{
				rows[i] = new KarrayList();
				columns[i] = new KarrayList();
				for (int q = 1; q < 10; q++)
				{
					rows[i].add(q);
					columns[i].add(q);
				}
		}
		
		for (int i = 0; i < 9; i++)
		{
			doRow(i);
		}
		setGivens();
	}
	
	public void setGivens()
	{
		int[][] temp = new int[9][9];
		int x, y, value;
		int numSolutions = 1;
		Point currPoint;
		ArrayList<Point> points = new ArrayList();
		
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
			{
				temp[j][i] = numbers[j][i].getValue();
			}
		
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
				points.add(new Point(j, i));

		Point p;
		for (int i = 0; i < 9; i++)
		{
			p = new Point(i, rand.nextInt(9));
			points.remove(p);
			temp[p.x][p.y] = 0;
		}
		
		do
		{
			currPoint = points.remove(rand.nextInt(points.size()));
			x = currPoint.x;
			y = currPoint.y;
			value = temp[x][y];
			temp[x][y] = 0;
			numSolutions = numSolutions(currPoint, temp);
			if (numSolutions > 1)
			{
				temp[x][y]=value;
			}
		}while(numSolutions == 1);
		
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
			{
				if (temp[j][i] == 0)
					numbers[j][i].showValue(false);				
			}

	}
	
	public int numSolutions (Point p, int[][] puzzle)
	{
		ArrayList<Integer> vals = new ArrayList(getPossibleValue(p, puzzle));
		Point nextPoint = new Point();
		int solutions = 0;
		if (vals.size() < 0)
			return 0;
		for (Integer i:vals)
		{
			puzzle[p.x][p.y]=i;
			nextPoint = leastValidPoint(puzzle);
			if (nextPoint.x == -1)
				solutions++;
			else
				solutions+=numSolutions(nextPoint, puzzle);
			if (solutions > 1)
			{
				puzzle[p.x][p.y]=0;
				return solutions;
			}
		}
		puzzle[p.x][p.y] = 0;
		return solutions;		
	}	
	
	public Point leastValidPoint(int[][] puzzle)
	{
		Point leastValidPoint = new Point(-1,-1);
		ArrayList<Integer> leastValidArray = new ArrayList();
		ArrayList<Integer> temp;
		for (int i = 0; i < 9; i++)
			leastValidArray.add(i);
			
		for (int i = 0; i < 9; i++)
			for (int j = 0; j < 9; j++)
				if (puzzle[j][i]==0)
				{
					temp = getPossibleValue(new Point(j,i), puzzle);
					if(temp.size() < leastValidArray.size())
					{
						leastValidArray = temp;
						leastValidPoint = new Point(j, i);
					}
				}			
				
		return leastValidPoint; 	
	}

	public ArrayList<Integer> getPossibleValue(Point p, int[][] puzzle)
	{
		int x = p.x;
		int y = p.y;
		ArrayList<Integer> quadrant = new ArrayList();
		ArrayList<Integer> row = new ArrayList();
		ArrayList<Integer> column = new ArrayList();
		ArrayList<Integer> Values;
		int quadx, quady;
		quadx = x/3;
		quady = y/3;
		
		for (int i = 1; i < 10; i++)
		{
			quadrant.add(i);
			row.add(i);
			column.add(i);
		}
		
		for (int i = 0; i < 9; i++)
		{
			if (puzzle[i][y]!=0)
				row.remove(row.indexOf(puzzle[i][y]));
			
			if (puzzle[x][i]!=0)
				column.remove(column.indexOf(puzzle[x][i]));			
		}
		
		for (int i = quady*3; i < quadx*3+3; i++)
		{
			for (int j = quadx*3; j < quady*3+3; j++)
			{
				if (puzzle[j][i] != 0)
					quadrant.remove(quadrant.indexOf(puzzle[j][i]));
			}
		}
		
		Values = new ArrayList(quadrant);
		Values.retainAll(row);
		Values.retainAll(column);
		
		return Values;
	}

	public void doRow(int i)
	{
			int num;	
			ArrayList<Integer> nums = new ArrayList();
			for (int j = 0; j < 9; j++)
			{
				num = getNum(j,i);
				if(num==0)
				{
					ArrayList<Integer> quadrant;
					ArrayList<Integer> column;
					ArrayList<Integer> row;
						
					for (int q = 0; q < j; q++)
					{
						int quadx, quady;
						quadx = q/3;
						quady = i/3;
						quadrant = quadrants[quadx][quady];
						column = columns[q];
						row = rows[i];
						quadrant.add(nums.get(q));
						column.add(nums.get(q));
						row.add(nums.get(q));		
					}
				
					if(overflow <= 400)
					{
						overflow++;
						doRow(i);
						return;
					}
					else
					{	
						overflow = 0;
						makeGame();
						return;
					}
				}
				else
				{
				nums.add(num);					
				if ((j+1)%3 == 0 && (i+1)%3==0)	
					{
						numbers[j][i] = new Number(num, 2);
					}
				else 
					if ((i+1)%3 == 0)
					{
						numbers[j][i] = new Number(num , 1);
					}
				else
				   if (((j+1)%3)==0)
					{
						numbers[j][i] = new Number(num , 0);
					}	
				else
					{
						numbers[j][i] = new Number(num);
					}
				}
			}
	}

	
	
	public int fact(int num)
	{
		int ret = 1;
		for (int i = num; i > 0; i--)
			ret = ret*i;
		
		return ret;
	}
	public int getNum(int x, int y)
	{
		Random rand = new Random();
		int quadx, quady;
		quadx = x/3;
		quady = y/3;
		ArrayList<Integer> quadrant = quadrants[quadx][quady];
		ArrayList<Integer> column = columns[x];
		ArrayList<Integer> row = rows[y];
		ArrayList<Integer> columnNumbers, rowNumbers, numbers;
		numbers = new ArrayList(quadrant);
		columnNumbers = column;
		rowNumbers = row;
						
		numbers.retainAll(columnNumbers);
		numbers.retainAll(rowNumbers);
		if (numbers.size() > 0)
		{	
			Integer numb = numbers.get(rand.nextInt(numbers.size()));
			row.remove(row.indexOf(numb));
			column.remove(column.indexOf(numb));
			quadrant.remove(quadrant.indexOf(numb));
			return numb;
		}
		else
			return 0;
	}
	
	public void removeNum(int x, int y, int num)
	{
		int quadx, quady;
		quadx = x/3;
		quady = y/3;
		ArrayList<Integer> quadrant = quadrants[quadx][quady];
		ArrayList<Integer> column = columns[x];
		ArrayList<Integer> row = rows[y];
	}
	
	public Number[][] getNumbers()
	{
		return numbers;		
	}
	
	
}